@@ -14,32 +14,48 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import Optional
+from __future__ import annotations
+
+from http import HTTPStatus
 
 import pendulum
 from connexion import NoContent
-from flask import current_app, g, request
+from flask import g
 from marshmallow import ValidationError
 from sqlalchemy import or_
+from sqlalchemy.orm import Query, Session
 
-from airflow.api.common.experimental.mark_tasks import (
+from airflow.api.common.mark_tasks import (
     set_dag_run_state_to_failed,
+    set_dag_run_state_to_queued,
     set_dag_run_state_to_success,
 )
 from airflow.api_connexion import security
+from airflow.api_connexion.endpoints.request_dict import get_json_request_dict
 from airflow.api_connexion.exceptions import AlreadyExists, BadRequest, NotFound
 from airflow.api_connexion.parameters import apply_sorting, check_limit, format_datetime, format_parameters
 from airflow.api_connexion.schemas.dag_run_schema import (
     DAGRunCollection,
+    clear_dagrun_form_schema,
     dagrun_collection_schema,
     dagrun_schema,
     dagruns_batch_form_schema,
     set_dagrun_state_form_schema,
 )
+from airflow.api_connexion.schemas.dataset_schema import (
+    DatasetEventCollection,
+    dataset_event_collection_schema,
+)
+from airflow.api_connexion.schemas.task_instance_schema import (
+    TaskInstanceReferenceCollection,
+    task_instance_reference_collection_schema,
+)
+from airflow.api_connexion.types import APIResponse
 from airflow.models import DagModel, DagRun
 from airflow.security import permissions
-from airflow.utils.session import provide_session
-from airflow.utils.state import State
+from airflow.utils.airflow_flask_app import get_airflow_app
+from airflow.utils.session import NEW_SESSION, provide_session
+from airflow.utils.state import DagRunState
 from airflow.utils.types import DagRunType
 
 
@@ -47,24 +63,24 @@ from airflow.utils.types import DagRunType
     [
         (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_DELETE, permissions.RESOURCE_DAG_RUN),
-    ]
+    ],
 )
 @provide_session
-def delete_dag_run(dag_id, dag_run_id, session):
+def delete_dag_run(*, dag_id: str, dag_run_id: str, session: Session = NEW_SESSION) -> APIResponse:
     """Delete a DAG Run"""
     if session.query(DagRun).filter(DagRun.dag_id == dag_id, DagRun.run_id == dag_run_id).delete() == 0:
         raise NotFound(detail=f"DAGRun with DAG ID: '{dag_id}' and DagRun ID: '{dag_run_id}' not found")
-    return NoContent, 204
+    return NoContent, HTTPStatus.NO_CONTENT
 
 
 @security.requires_access(
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
-    ]
+    ],
 )
 @provide_session
-def get_dag_run(dag_id, dag_run_id, session):
+def get_dag_run(*, dag_id: str, dag_run_id: str, session: Session = NEW_SESSION) -> APIResponse:
     """Get a DAG Run."""
     dag_run = session.query(DagRun).filter(DagRun.dag_id == dag_id, DagRun.run_id == dag_run_id).one_or_none()
     if dag_run is None:
@@ -79,105 +95,46 @@ def get_dag_run(dag_id, dag_run_id, session):
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
-    ]
-)
-@format_parameters(
-    {
-        'start_date_gte': format_datetime,
-        'start_date_lte': format_datetime,
-        'execution_date_gte': format_datetime,
-        'execution_date_lte': format_datetime,
-        'end_date_gte': format_datetime,
-        'end_date_lte': format_datetime,
-        'limit': check_limit,
-    }
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DATASET),
+    ],
 )
 @provide_session
-def get_dag_runs(
-    session,
-    dag_id,
-    start_date_gte=None,
-    start_date_lte=None,
-    execution_date_gte=None,
-    execution_date_lte=None,
-    end_date_gte=None,
-    end_date_lte=None,
-    offset=None,
-    limit=None,
-    order_by='id',
-):
-    """Get all DAG Runs."""
-    query = session.query(DagRun)
-
-    #  This endpoint allows specifying ~ as the dag_id to retrieve DAG Runs for all DAGs.
-    if dag_id == "~":
-        appbuilder = current_app.appbuilder
-        query = query.filter(DagRun.dag_id.in_(appbuilder.sm.get_readable_dag_ids(g.user)))
-    else:
-        query = query.filter(DagRun.dag_id == dag_id)
-
-    dag_run, total_entries = _fetch_dag_runs(
-        query,
-        end_date_gte,
-        end_date_lte,
-        execution_date_gte,
-        execution_date_lte,
-        start_date_gte,
-        start_date_lte,
-        limit,
-        offset,
-        order_by,
+def get_upstream_dataset_events(
+    *, dag_id: str, dag_run_id: str, session: Session = NEW_SESSION
+) -> APIResponse:
+    """If dag run is dataset-triggered, return the dataset events that triggered it."""
+    dag_run: DagRun | None = (
+        session.query(DagRun)
+        .filter(
+            DagRun.dag_id == dag_id,
+            DagRun.run_id == dag_run_id,
+        )
+        .one_or_none()
+    )
+    if dag_run is None:
+        raise NotFound(
+            "DAGRun not found",
+            detail=f"DAGRun with DAG ID: '{dag_id}' and DagRun ID: '{dag_run_id}' not found",
+        )
+    events = dag_run.consumed_dataset_events
+    return dataset_event_collection_schema.dump(
+        DatasetEventCollection(dataset_events=events, total_entries=len(events))
     )
-
-    return dagrun_collection_schema.dump(DAGRunCollection(dag_runs=dag_run, total_entries=total_entries))
 
 
 def _fetch_dag_runs(
-    query,
-    end_date_gte,
-    end_date_lte,
-    execution_date_gte,
-    execution_date_lte,
-    start_date_gte,
-    start_date_lte,
-    limit,
-    offset,
-    order_by,
-):
-    query = _apply_date_filters_to_query(
-        query,
-        end_date_gte,
-        end_date_lte,
-        execution_date_gte,
-        execution_date_lte,
-        start_date_gte,
-        start_date_lte,
-    )
-    # Count items
-    total_entries = query.count()
-    # sort
-    to_replace = {"dag_run_id": "run_id"}
-    allowed_filter_attrs = [
-        "id",
-        "state",
-        "dag_id",
-        "execution_date",
-        "dag_run_id",
-        "start_date",
-        "end_date",
-        "external_trigger",
-        "conf",
-    ]
-    query = apply_sorting(query, order_by, to_replace, allowed_filter_attrs)
-    # apply offset and limit
-    dag_run = query.offset(offset).limit(limit).all()
-    return dag_run, total_entries
-
-
-def _apply_date_filters_to_query(
-    query, end_date_gte, end_date_lte, execution_date_gte, execution_date_lte, start_date_gte, start_date_lte
-):
-    # filter start date
+    query: Query,
+    *,
+    end_date_gte: str | None,
+    end_date_lte: str | None,
+    execution_date_gte: str | None,
+    execution_date_lte: str | None,
+    start_date_gte: str | None,
+    start_date_lte: str | None,
+    limit: int | None,
+    offset: int | None,
+    order_by: str,
+) -> tuple[list[DagRun], int]:
     if start_date_gte:
         query = query.filter(DagRun.start_date >= start_date_gte)
     if start_date_lte:
@@ -192,25 +149,101 @@ def _apply_date_filters_to_query(
         query = query.filter(DagRun.end_date >= end_date_gte)
     if end_date_lte:
         query = query.filter(DagRun.end_date <= end_date_lte)
-    return query
+
+    total_entries = query.count()
+    to_replace = {"dag_run_id": "run_id"}
+    allowed_filter_attrs = [
+        "id",
+        "state",
+        "dag_id",
+        "execution_date",
+        "dag_run_id",
+        "start_date",
+        "end_date",
+        "external_trigger",
+        "conf",
+    ]
+    query = apply_sorting(query, order_by, to_replace, allowed_filter_attrs)
+    return query.offset(offset).limit(limit).all(), total_entries
 
 
 @security.requires_access(
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
-    ]
+    ],
+)
+@format_parameters(
+    {
+        'start_date_gte': format_datetime,
+        'start_date_lte': format_datetime,
+        'execution_date_gte': format_datetime,
+        'execution_date_lte': format_datetime,
+        'end_date_gte': format_datetime,
+        'end_date_lte': format_datetime,
+        'limit': check_limit,
+    }
 )
 @provide_session
-def get_dag_runs_batch(session):
+def get_dag_runs(
+    *,
+    dag_id: str,
+    start_date_gte: str | None = None,
+    start_date_lte: str | None = None,
+    execution_date_gte: str | None = None,
+    execution_date_lte: str | None = None,
+    end_date_gte: str | None = None,
+    end_date_lte: str | None = None,
+    state: list[str] | None = None,
+    offset: int | None = None,
+    limit: int | None = None,
+    order_by: str = "id",
+    session: Session = NEW_SESSION,
+):
+    """Get all DAG Runs."""
+    query = session.query(DagRun)
+
+    #  This endpoint allows specifying ~ as the dag_id to retrieve DAG Runs for all DAGs.
+    if dag_id == "~":
+        appbuilder = get_airflow_app().appbuilder
+        query = query.filter(DagRun.dag_id.in_(appbuilder.sm.get_readable_dag_ids(g.user)))
+    else:
+        query = query.filter(DagRun.dag_id == dag_id)
+
+    if state:
+        query = query.filter(DagRun.state.in_(state))
+
+    dag_run, total_entries = _fetch_dag_runs(
+        query,
+        end_date_gte=end_date_gte,
+        end_date_lte=end_date_lte,
+        execution_date_gte=execution_date_gte,
+        execution_date_lte=execution_date_lte,
+        start_date_gte=start_date_gte,
+        start_date_lte=start_date_lte,
+        limit=limit,
+        offset=offset,
+        order_by=order_by,
+    )
+    return dagrun_collection_schema.dump(DAGRunCollection(dag_runs=dag_run, total_entries=total_entries))
+
+
+@security.requires_access(
+    [
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG_RUN),
+    ],
+)
+@provide_session
+def get_dag_runs_batch(*, session: Session = NEW_SESSION) -> APIResponse:
     """Get list of DAG Runs"""
-    body = request.get_json()
+    body = get_json_request_dict()
     try:
         data = dagruns_batch_form_schema.load(body)
     except ValidationError as err:
         raise BadRequest(detail=str(err.messages))
 
-    appbuilder = current_app.appbuilder
+    appbuilder = get_airflow_app().appbuilder
     readable_dag_ids = appbuilder.sm.get_readable_dag_ids(g.user)
     query = session.query(DagRun)
     if data.get("dag_ids"):
@@ -219,17 +252,21 @@ def get_dag_runs_batch(session):
     else:
         query = query.filter(DagRun.dag_id.in_(readable_dag_ids))
 
+    states = data.get("states")
+    if states:
+        query = query.filter(DagRun.state.in_(states))
+
     dag_runs, total_entries = _fetch_dag_runs(
         query,
-        data["end_date_gte"],
-        data["end_date_lte"],
-        data["execution_date_gte"],
-        data["execution_date_lte"],
-        data["start_date_gte"],
-        data["start_date_lte"],
-        data["page_limit"],
-        data["page_offset"],
-        order_by=data.get('order_by', "id"),
+        end_date_gte=data["end_date_gte"],
+        end_date_lte=data["end_date_lte"],
+        execution_date_gte=data["execution_date_gte"],
+        execution_date_lte=data["execution_date_lte"],
+        start_date_gte=data["start_date_gte"],
+        start_date_lte=data["start_date_lte"],
+        limit=data["page_limit"],
+        offset=data["page_offset"],
+        order_by=data.get("order_by", "id"),
     )
 
     return dagrun_collection_schema.dump(DAGRunCollection(dag_runs=dag_runs, total_entries=total_entries))
@@ -239,10 +276,10 @@ def get_dag_runs_batch(session):
     [
         (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_CREATE, permissions.RESOURCE_DAG_RUN),
-    ]
+    ],
 )
 @provide_session
-def post_dag_run(dag_id, session):
+def post_dag_run(*, dag_id: str, session: Session = NEW_SESSION) -> APIResponse:
     """Trigger a DAG."""
     dm = session.query(DagModel).filter(DagModel.dag_id == dag_id).first()
     if not dm:
@@ -253,7 +290,7 @@ def post_dag_run(dag_id, session):
             detail=f"DAG with dag_id: '{dag_id}' has import errors",
         )
     try:
-        post_body = dagrun_schema.load(request.json, session=session)
+        post_body = dagrun_schema.load(get_json_request_dict(), session=session)
     except ValidationError as err:
         raise BadRequest(detail=str(err))
 
@@ -269,16 +306,16 @@ def post_dag_run(dag_id, session):
     )
     if not dagrun_instance:
         try:
-            dag = current_app.dag_bag.get_dag(dag_id)
+            dag = get_airflow_app().dag_bag.get_dag(dag_id)
             dag_run = dag.create_dagrun(
                 run_type=DagRunType.MANUAL,
                 run_id=run_id,
                 execution_date=logical_date,
                 data_interval=dag.timetable.infer_manual_data_interval(run_after=logical_date),
-                state=State.QUEUED,
+                state=DagRunState.QUEUED,
                 conf=post_body.get("conf"),
                 external_trigger=True,
-                dag_hash=current_app.dag_bag.dags_hash.get(dag_id),
+                dag_hash=get_airflow_app().dag_bag.dags_hash.get(dag_id),
             )
             return dagrun_schema.dump(dag_run)
         except ValueError as ve:
@@ -299,27 +336,80 @@ def post_dag_run(dag_id, session):
     [
         (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
         (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG_RUN),
-    ]
+    ],
 )
 @provide_session
-def update_dag_run_state(dag_id: str, dag_run_id: str, session) -> dict:
+def update_dag_run_state(*, dag_id: str, dag_run_id: str, session: Session = NEW_SESSION) -> APIResponse:
     """Set a state of a dag run."""
-    dag_run: Optional[DagRun] = (
+    dag_run: DagRun | None = (
         session.query(DagRun).filter(DagRun.dag_id == dag_id, DagRun.run_id == dag_run_id).one_or_none()
     )
     if dag_run is None:
         error_message = f'Dag Run id {dag_run_id} not found in dag {dag_id}'
         raise NotFound(error_message)
     try:
-        post_body = set_dagrun_state_form_schema.load(request.json)
+        post_body = set_dagrun_state_form_schema.load(get_json_request_dict())
     except ValidationError as err:
         raise BadRequest(detail=str(err))
 
     state = post_body['state']
-    dag = current_app.dag_bag.get_dag(dag_id)
-    if state == State.SUCCESS:
-        set_dag_run_state_to_success(dag, dag_run.execution_date, commit=True)
+    dag = get_airflow_app().dag_bag.get_dag(dag_id)
+    if state == DagRunState.SUCCESS:
+        set_dag_run_state_to_success(dag=dag, run_id=dag_run.run_id, commit=True)
+    elif state == DagRunState.QUEUED:
+        set_dag_run_state_to_queued(dag=dag, run_id=dag_run.run_id, commit=True)
     else:
-        set_dag_run_state_to_failed(dag, dag_run.execution_date, commit=True)
+        set_dag_run_state_to_failed(dag=dag, run_id=dag_run.run_id, commit=True)
     dag_run = session.query(DagRun).get(dag_run.id)
     return dagrun_schema.dump(dag_run)
+
+
+@security.requires_access(
+    [
+        (permissions.ACTION_CAN_READ, permissions.RESOURCE_DAG),
+        (permissions.ACTION_CAN_EDIT, permissions.RESOURCE_DAG_RUN),
+    ],
+)
+@provide_session
+def clear_dag_run(*, dag_id: str, dag_run_id: str, session: Session = NEW_SESSION) -> APIResponse:
+    """Clear a dag run."""
+    dag_run: DagRun | None = (
+        session.query(DagRun).filter(DagRun.dag_id == dag_id, DagRun.run_id == dag_run_id).one_or_none()
+    )
+    if dag_run is None:
+        error_message = f'Dag Run id {dag_run_id} not found in dag   {dag_id}'
+        raise NotFound(error_message)
+    try:
+        post_body = clear_dagrun_form_schema.load(get_json_request_dict())
+    except ValidationError as err:
+        raise BadRequest(detail=str(err))
+
+    dry_run = post_body.get('dry_run', False)
+    dag = get_airflow_app().dag_bag.get_dag(dag_id)
+    start_date = dag_run.logical_date
+    end_date = dag_run.logical_date
+
+    if dry_run:
+        task_instances = dag.clear(
+            start_date=start_date,
+            end_date=end_date,
+            task_ids=None,
+            include_subdags=True,
+            include_parentdag=True,
+            only_failed=False,
+            dry_run=True,
+        )
+        return task_instance_reference_collection_schema.dump(
+            TaskInstanceReferenceCollection(task_instances=task_instances)
+        )
+    else:
+        dag.clear(
+            start_date=start_date,
+            end_date=end_date,
+            task_ids=None,
+            include_subdags=True,
+            include_parentdag=True,
+            only_failed=False,
+        )
+        dag_run.refresh_from_db()
+        return dagrun_schema.dump(dag_run)
