@@ -14,9 +14,10 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import json
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from airflow.models.taskinstance import SimpleTaskInstance
@@ -28,10 +29,17 @@ class CallbackRequest:
 
     :param full_filepath: File Path to use to run the callback
     :param msg: Additional Message that can be used for logging
+    :param processor_subdir: Directory used by Dag Processor when parsed the dag.
     """
 
-    def __init__(self, full_filepath: str, msg: Optional[str] = None):
+    def __init__(
+        self,
+        full_filepath: str,
+        processor_subdir: str | None = None,
+        msg: str | None = None,
+    ):
         self.full_filepath = full_filepath
+        self.processor_subdir = processor_subdir
         self.msg = msg
 
     def __eq__(self, other):
@@ -60,16 +68,18 @@ class TaskCallbackRequest(CallbackRequest):
     :param simple_task_instance: Simplified Task Instance representation
     :param is_failure_callback: Flag to determine whether it is a Failure Callback or Success Callback
     :param msg: Additional Message that can be used for logging to determine failure/zombie
+    :param processor_subdir: Directory used by Dag Processor when parsed the dag.
     """
 
     def __init__(
         self,
         full_filepath: str,
-        simple_task_instance: "SimpleTaskInstance",
-        is_failure_callback: Optional[bool] = True,
-        msg: Optional[str] = None,
+        simple_task_instance: SimpleTaskInstance,
+        is_failure_callback: bool | None = True,
+        processor_subdir: str | None = None,
+        msg: str | None = None,
     ):
-        super().__init__(full_filepath=full_filepath, msg=msg)
+        super().__init__(full_filepath=full_filepath, processor_subdir=processor_subdir, msg=msg)
         self.simple_task_instance = simple_task_instance
         self.is_failure_callback = is_failure_callback
 
@@ -94,6 +104,7 @@ class DagCallbackRequest(CallbackRequest):
     :param full_filepath: File Path to use to run the callback
     :param dag_id: DAG ID
     :param run_id: Run ID for the DagRun
+    :param processor_subdir: Directory used by Dag Processor when parsed the dag.
     :param is_failure_callback: Flag to determine whether it is a Failure Callback or Success Callback
     :param msg: Additional Message that can be used for logging
     """
@@ -103,10 +114,11 @@ class DagCallbackRequest(CallbackRequest):
         full_filepath: str,
         dag_id: str,
         run_id: str,
-        is_failure_callback: Optional[bool] = True,
-        msg: Optional[str] = None,
+        processor_subdir: str | None,
+        is_failure_callback: bool | None = True,
+        msg: str | None = None,
     ):
-        super().__init__(full_filepath=full_filepath, msg=msg)
+        super().__init__(full_filepath=full_filepath, processor_subdir=processor_subdir, msg=msg)
         self.dag_id = dag_id
         self.run_id = run_id
         self.is_failure_callback = is_failure_callback
@@ -118,8 +130,15 @@ class SlaCallbackRequest(CallbackRequest):
 
     :param full_filepath: File Path to use to run the callback
     :param dag_id: DAG ID
+    :param processor_subdir: Directory used by Dag Processor when parsed the dag.
     """
 
-    def __init__(self, full_filepath: str, dag_id: str, msg: Optional[str] = None):
-        super().__init__(full_filepath, msg)
+    def __init__(
+        self,
+        full_filepath: str,
+        dag_id: str,
+        processor_subdir: str | None,
+        msg: str | None = None,
+    ):
+        super().__init__(full_filepath, processor_subdir=processor_subdir, msg=msg)
         self.dag_id = dag_id
