@@ -15,64 +15,90 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """
 Example DAG demonstrating the usage of DateTimeBranchOperator with datetime as well as time objects as
 targets.
 """
-import datetime
+from __future__ import annotations
+
+import pendulum
 
 from airflow import DAG
 from airflow.operators.datetime import BranchDateTimeOperator
-from airflow.operators.dummy import DummyOperator
+from airflow.operators.empty import EmptyOperator
 
-dag = DAG(
+dag1 = DAG(
     dag_id="example_branch_datetime_operator",
-    start_date=datetime.datetime(2021, 1, 1),
+    start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
     catchup=False,
     tags=["example"],
-    schedule_interval="@daily",
+    schedule="@daily",
 )
 
 # [START howto_branch_datetime_operator]
-dummy_task_1 = DummyOperator(task_id='date_in_range', dag=dag)
-dummy_task_2 = DummyOperator(task_id='date_outside_range', dag=dag)
+empty_task_11 = EmptyOperator(task_id='date_in_range', dag=dag1)
+empty_task_21 = EmptyOperator(task_id='date_outside_range', dag=dag1)
 
 cond1 = BranchDateTimeOperator(
     task_id='datetime_branch',
     follow_task_ids_if_true=['date_in_range'],
     follow_task_ids_if_false=['date_outside_range'],
-    target_upper=datetime.datetime(2020, 10, 10, 15, 0, 0),
-    target_lower=datetime.datetime(2020, 10, 10, 14, 0, 0),
-    dag=dag,
+    target_upper=pendulum.datetime(2020, 10, 10, 15, 0, 0),
+    target_lower=pendulum.datetime(2020, 10, 10, 14, 0, 0),
+    dag=dag1,
 )
 
-# Run dummy_task_1 if cond1 executes between 2020-10-10 14:00:00 and 2020-10-10 15:00:00
-cond1 >> [dummy_task_1, dummy_task_2]
+# Run empty_task_1 if cond1 executes between 2020-10-10 14:00:00 and 2020-10-10 15:00:00
+cond1 >> [empty_task_11, empty_task_21]
 # [END howto_branch_datetime_operator]
 
 
-dag = DAG(
+dag2 = DAG(
     dag_id="example_branch_datetime_operator_2",
-    start_date=datetime.datetime(2021, 1, 1),
+    start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
     catchup=False,
     tags=["example"],
-    schedule_interval="@daily",
+    schedule="@daily",
 )
 # [START howto_branch_datetime_operator_next_day]
-dummy_task_1 = DummyOperator(task_id='date_in_range', dag=dag)
-dummy_task_2 = DummyOperator(task_id='date_outside_range', dag=dag)
+empty_task_12 = EmptyOperator(task_id='date_in_range', dag=dag2)
+empty_task_22 = EmptyOperator(task_id='date_outside_range', dag=dag2)
 
 cond2 = BranchDateTimeOperator(
     task_id='datetime_branch',
     follow_task_ids_if_true=['date_in_range'],
     follow_task_ids_if_false=['date_outside_range'],
-    target_upper=datetime.time(0, 0, 0),
-    target_lower=datetime.time(15, 0, 0),
-    dag=dag,
+    target_upper=pendulum.time(0, 0, 0),
+    target_lower=pendulum.time(15, 0, 0),
+    dag=dag2,
 )
 
 # Since target_lower happens after target_upper, target_upper will be moved to the following day
-# Run dummy_task_1 if cond2 executes between 15:00:00, and 00:00:00 of the following day
-cond2 >> [dummy_task_1, dummy_task_2]
+# Run empty_task_1 if cond2 executes between 15:00:00, and 00:00:00 of the following day
+cond2 >> [empty_task_12, empty_task_22]
 # [END howto_branch_datetime_operator_next_day]
+
+dag3 = DAG(
+    dag_id="example_branch_datetime_operator_3",
+    start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
+    catchup=False,
+    tags=["example"],
+    schedule="@daily",
+)
+# [START howto_branch_datetime_operator_logical_date]
+empty_task_13 = EmptyOperator(task_id='date_in_range', dag=dag3)
+empty_task_23 = EmptyOperator(task_id='date_outside_range', dag=dag3)
+
+cond3 = BranchDateTimeOperator(
+    task_id='datetime_branch',
+    use_task_logical_date=True,
+    follow_task_ids_if_true=['date_in_range'],
+    follow_task_ids_if_false=['date_outside_range'],
+    target_upper=pendulum.datetime(2020, 10, 10, 15, 0, 0),
+    target_lower=pendulum.datetime(2020, 10, 10, 14, 0, 0),
+    dag=dag3,
+)
+
+# Run empty_task_3 if cond1 executes between 2020-10-10 14:00:00 and 2020-10-10 15:00:00
+cond3 >> [empty_task_13, empty_task_23]
+# [END howto_branch_datetime_operator_logical_date]
