@@ -15,13 +15,14 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
+from __future__ import annotations
 
 import time
-from collections import OrderedDict
-from typing import Optional, Set
+from typing import TYPE_CHECKING, Any, Iterable, Iterator, Sequence
 
+import attr
 import pendulum
+from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.session import Session, make_transient
 from tabulate import tabulate
 
@@ -45,9 +46,12 @@ from airflow.timetables.base import DagRunInfo
 from airflow.utils import helpers, timezone
 from airflow.utils.configuration import conf as airflow_conf, tmp_configuration_copy
 from airflow.utils.session import provide_session
-from airflow.utils.state import State
+from airflow.utils.state import DagRunState, State, TaskInstanceState
 from airflow.utils.types import DagRunType
 
+if TYPE_CHECKING:
+    from airflow.models.mappedoperator import MappedOperator
+
 
 class BackfillJob(BaseJob):
     """
@@ -60,6 +64,7 @@ class BackfillJob(BaseJob):
 
     __mapper_args__ = {'polymorphic_identity': 'BackfillJob'}
 
+    @attr.define
     class _DagRunTaskStatus:
         """
         Internal status of the backfill job. This class is intended to be instantiated
@@ -71,59 +76,33 @@ class BackfillJob(BaseJob):
         it easier to pass it around.
 
         :param to_run: Tasks to run in the backfill
-        :type to_run: dict[tuple[TaskInstanceKey], airflow.models.TaskInstance]
         :param running: Maps running task instance key to task instance object
-        :type running: dict[tuple[TaskInstanceKey], airflow.models.TaskInstance]
         :param skipped: Tasks that have been skipped
-        :type skipped: set[tuple[TaskInstanceKey]]
         :param succeeded: Tasks that have succeeded so far
-        :type succeeded: set[tuple[TaskInstanceKey]]
         :param failed: Tasks that have failed
-        :type failed: set[tuple[TaskInstanceKey]]
         :param not_ready: Tasks not ready for execution
-        :type not_ready: set[tuple[TaskInstanceKey]]
         :param deadlocked: Deadlocked tasks
-        :type deadlocked: set[airflow.models.TaskInstance]
         :param active_runs: Active dag runs at a certain point in time
-        :type active_runs: list[DagRun]
         :param executed_dag_run_dates: Datetime objects for the executed dag runs
-        :type executed_dag_run_dates: set[datetime.datetime]
         :param finished_runs: Number of finished runs so far
-        :type finished_runs: int
         :param total_runs: Number of total dag runs able to run
-        :type total_runs: int
         """
 
-        # TODO(edgarRd): AIRFLOW-1444: Add consistency check on counts
-        def __init__(
-            self,
-            to_run=None,
-            running=None,
-            skipped=None,
-            succeeded=None,
-            failed=None,
-            not_ready=None,
-            deadlocked=None,
-            active_runs=None,
-            executed_dag_run_dates=None,
-            finished_runs=0,
-            total_runs=0,
-        ):
-            self.to_run = to_run or OrderedDict()
-            self.running = running or {}
-            self.skipped = skipped or set()
-            self.succeeded = succeeded or set()
-            self.failed = failed or set()
-            self.not_ready = not_ready or set()
-            self.deadlocked = deadlocked or set()
-            self.active_runs = active_runs or []
-            self.executed_dag_run_dates = executed_dag_run_dates or set()
-            self.finished_runs = finished_runs
-            self.total_runs = total_runs
+        to_run: dict[TaskInstanceKey, TaskInstance] = attr.ib(factory=dict)
+        running: dict[TaskInstanceKey, TaskInstance] = attr.ib(factory=dict)
+        skipped: set[TaskInstanceKey] = attr.ib(factory=set)
+        succeeded: set[TaskInstanceKey] = attr.ib(factory=set)
+        failed: set[TaskInstanceKey] = attr.ib(factory=set)
+        not_ready: set[TaskInstanceKey] = attr.ib(factory=set)
+        deadlocked: set[TaskInstance] = attr.ib(factory=set)
+        active_runs: list[DagRun] = attr.ib(factory=list)
+        executed_dag_run_dates: set[pendulum.DateTime] = attr.ib(factory=set)
+        finished_runs: int = 0
+        total_runs: int = 0
 
     def __init__(
         self,
-        dag,
+        dag: DAG,
         start_date=None,
         end_date=None,
         mark_success=False,
@@ -137,39 +116,27 @@ class BackfillJob(BaseJob):
         rerun_failed_tasks=False,
         run_backwards=False,
         run_at_least_once=False,
+        continue_on_failures=False,
         *args,
         **kwargs,
     ):
         """
         :param dag: DAG object.
-        :type dag: airflow.models.DAG
         :param start_date: start date for the backfill date range.
-        :type start_date: datetime.datetime
         :param end_date: end date for the backfill date range.
-        :type end_date: datetime.datetime
         :param mark_success: flag whether to mark the task auto success.
-        :type mark_success: bool
         :param donot_pickle: whether pickle
-        :type donot_pickle: bool
         :param ignore_first_depends_on_past: whether to ignore depend on past
-        :type ignore_first_depends_on_past: bool
         :param ignore_task_deps: whether to ignore the task dependency
-        :type ignore_task_deps: bool
         :param pool: pool to backfill
-        :type pool: str
         :param delay_on_limit_secs:
         :param verbose:
-        :type verbose: flag to whether display verbose message to backfill console
         :param conf: a dictionary which user could pass k-v pairs for backfill
-        :type conf: dictionary
         :param rerun_failed_tasks: flag to whether to
                                    auto rerun the failed task in backfill
-        :type rerun_failed_tasks: bool
         :param run_backwards: Whether to process the dates from most to least recent
-        :type run_backwards bool
         :param run_at_least_once: If true, always run the DAG at least once even
             if no logical run exists within the time range.
-        :type: bool
         :param args:
         :param kwargs:
         """
@@ -188,16 +155,15 @@ class BackfillJob(BaseJob):
         self.rerun_failed_tasks = rerun_failed_tasks
         self.run_backwards = run_backwards
         self.run_at_least_once = run_at_least_once
+        self.continue_on_failures = continue_on_failures
         super().__init__(*args, **kwargs)
 
-    @provide_session
     def _update_counters(self, ti_status, session=None):
         """
         Updates the counters per state of the tasks that were running. Can re-add
         to tasks to run in case required.
 
         :param ti_status: the internal status of the backfill job tasks
-        :type ti_status: BackfillJob._DagRunTaskStatus
         """
         tis_to_be_scheduled = []
         refreshed_tis = []
@@ -210,28 +176,28 @@ class BackfillJob(BaseJob):
         for ti in refreshed_tis:
             # Here we remake the key by subtracting 1 to match in memory information
             reduced_key = ti.key.reduced
-            if ti.state == State.SUCCESS:
+            if ti.state == TaskInstanceState.SUCCESS:
                 ti_status.succeeded.add(reduced_key)
                 self.log.debug("Task instance %s succeeded. Don't rerun.", ti)
                 ti_status.running.pop(reduced_key)
                 continue
-            if ti.state == State.SKIPPED:
+            if ti.state == TaskInstanceState.SKIPPED:
                 ti_status.skipped.add(reduced_key)
                 self.log.debug("Task instance %s skipped. Don't rerun.", ti)
                 ti_status.running.pop(reduced_key)
                 continue
-            if ti.state == State.FAILED:
+            if ti.state == TaskInstanceState.FAILED:
                 self.log.error("Task instance %s failed", ti)
                 ti_status.failed.add(reduced_key)
                 ti_status.running.pop(reduced_key)
                 continue
             # special case: if the task needs to run again put it back
-            if ti.state == State.UP_FOR_RETRY:
+            if ti.state == TaskInstanceState.UP_FOR_RETRY:
                 self.log.warning("Task instance %s is up for retry", ti)
                 ti_status.running.pop(reduced_key)
                 ti_status.to_run[ti.key] = ti
             # special case: if the task needs to be rescheduled put it back
-            elif ti.state == State.UP_FOR_RESCHEDULE:
+            elif ti.state == TaskInstanceState.UP_FOR_RESCHEDULE:
                 self.log.warning("Task instance %s is up for reschedule", ti)
                 # During handling of reschedule state in ti._handle_reschedule, try number is reduced
                 # by one, so we should not use reduced_key to avoid key error
@@ -256,15 +222,21 @@ class BackfillJob(BaseJob):
         if tis_to_be_scheduled:
             filter_for_tis = TI.filter_for_tis(tis_to_be_scheduled)
             session.query(TI).filter(filter_for_tis).update(
-                values={TI.state: State.SCHEDULED}, synchronize_session=False
+                values={TI.state: TaskInstanceState.SCHEDULED}, synchronize_session=False
             )
+            session.flush()
 
-    def _manage_executor_state(self, running):
+    def _manage_executor_state(
+        self, running, session
+    ) -> Iterator[tuple[MappedOperator, str, Sequence[TaskInstance], int]]:
         """
         Checks if the executor agrees with the state of task instances
-        that are running
+        that are running.
+
+        Expands downstream mapped tasks when necessary
 
         :param running: dict of key, task to verify
+        :return: An iterable of expanded TaskInstance per MappedTask
         """
         executor = self.executor
 
@@ -280,14 +252,23 @@ class BackfillJob(BaseJob):
 
             self.log.debug("Executor state: %s task %s", state, ti)
 
-            if state in (State.FAILED, State.SUCCESS) and ti.state in self.STATES_COUNT_AS_RUNNING:
+            if (
+                state in (TaskInstanceState.FAILED, TaskInstanceState.SUCCESS)
+                and ti.state in self.STATES_COUNT_AS_RUNNING
+            ):
                 msg = (
-                    "Executor reports task instance {} finished ({}) "
-                    "although the task says its {}. Was the task "
-                    "killed externally? Info: {}".format(ti, state, ti.state, info)
+                    f"Executor reports task instance {ti} finished ({state}) although the task says its "
+                    f"{ti.state}. Was the task killed externally? Info: {info}"
                 )
                 self.log.error(msg)
-                ti.handle_failure_with_callback(error=msg)
+                ti.handle_failure(error=msg)
+                continue
+            if ti.state not in self.STATES_COUNT_AS_RUNNING:
+                # Don't use ti.task; if this task is mapped, that attribute
+                # would hold the unmapped task. We need to original task here.
+                for node in self.dag.get_task(ti.task_id, include_subdags=True).iter_mapped_dependants():
+                    new_tis, num_mapped_tis = node.expand_mapped_task(ti.run_id, session=session)
+                    yield node, ti.run_id, new_tis, num_mapped_tis
 
     @provide_session
     def _get_dag_run(self, dagrun_info: DagRunInfo, dag: DAG, session: Session = None):
@@ -311,11 +292,15 @@ class BackfillJob(BaseJob):
         # check if we are scheduling on top of a already existing dag_run
         # we could find a "scheduled" run instead of a "backfill"
         runs = DagRun.find(dag_id=dag.dag_id, execution_date=run_date, session=session)
-        run: Optional[DagRun]
+        run: DagRun | None
         if runs:
             run = runs[0]
-            if run.state == State.RUNNING:
+            if run.state == DagRunState.RUNNING:
                 respect_dag_max_active_limit = False
+            # Fixes --conf overwrite for backfills with already existing DagRuns
+            run.conf = self.conf or {}
+            # start_date is cleared for existing DagRuns
+            run.start_date = timezone.utcnow()
         else:
             run = None
 
@@ -328,7 +313,7 @@ class BackfillJob(BaseJob):
             execution_date=run_date,
             data_interval=dagrun_info.data_interval,
             start_date=timezone.utcnow(),
-            state=State.RUNNING,
+            state=DagRunState.RUNNING,
             external_trigger=False,
             session=session,
             conf=self.conf,
@@ -340,21 +325,19 @@ class BackfillJob(BaseJob):
         run.dag = dag
 
         # explicitly mark as backfill and running
-        run.state = State.RUNNING
+        run.state = DagRunState.RUNNING
         run.run_type = DagRunType.BACKFILL_JOB
         run.verify_integrity(session=session)
         return run
 
     @provide_session
-    def _task_instances_for_dag_run(self, dag_run, session=None):
+    def _task_instances_for_dag_run(self, dag, dag_run, session=None):
         """
         Returns a map of task instance key to task instance object for the tasks to
         run in the given dag run.
 
         :param dag_run: the dag run to get the tasks from
-        :type dag_run: airflow.models.DagRun
         :param session: the database session object
-        :type session: sqlalchemy.orm.session.Session
         """
         tasks_to_run = {}
 
@@ -368,18 +351,19 @@ class BackfillJob(BaseJob):
         dag_run.refresh_from_db()
         make_transient(dag_run)
 
+        dag_run.dag = dag
+        info = dag_run.task_instance_scheduling_decisions(session=session)
+        schedulable_tis = info.schedulable_tis
         try:
-            for ti in dag_run.get_task_instances():
-                # all tasks part of the backfill are scheduled to run
-                if ti.state == State.NONE:
-                    ti.set_state(State.SCHEDULED, session=session)
-                if ti.state != State.REMOVED:
+            for ti in dag_run.get_task_instances(session=session):
+                if ti in schedulable_tis:
+                    ti.set_state(TaskInstanceState.SCHEDULED)
+                if ti.state != TaskInstanceState.REMOVED:
                     tasks_to_run[ti.key] = ti
             session.commit()
         except Exception:
             session.rollback()
             raise
-
         return tasks_to_run
 
     def _log_progress(self, ti_status):
@@ -414,15 +398,10 @@ class BackfillJob(BaseJob):
         them in a backfill process.
 
         :param ti_status: the internal status of the job
-        :type ti_status: BackfillJob._DagRunTaskStatus
         :param executor: the executor to run the task instances
-        :type executor: BaseExecutor
         :param pickle_id: the pickle_id if dag is pickled, None otherwise
-        :type pickle_id: int
         :param start_date: the start date of the backfill job
-        :type start_date: datetime.datetime
         :param session: the current session object
-        :type session: sqlalchemy.orm.session.Session
         :return: the list of execution_dates for the finished dag runs
         :rtype: list
         """
@@ -438,7 +417,6 @@ class BackfillJob(BaseJob):
             # or leaf to root, as otherwise tasks might be
             # determined deadlocked while they are actually
             # waiting for their upstream to finish
-            @provide_session
             def _per_task_process(key, ti: TaskInstance, session=None):
                 ti.refresh_from_db(lock_for_update=True, session=session)
 
@@ -449,14 +427,14 @@ class BackfillJob(BaseJob):
 
                 # The task was already marked successful or skipped by a
                 # different Job. Don't rerun it.
-                if ti.state == State.SUCCESS:
+                if ti.state == TaskInstanceState.SUCCESS:
                     ti_status.succeeded.add(key)
                     self.log.debug("Task instance %s succeeded. Don't rerun.", ti)
                     ti_status.to_run.pop(key)
                     if key in ti_status.running:
                         ti_status.running.pop(key)
                     return
-                elif ti.state == State.SKIPPED:
+                elif ti.state == TaskInstanceState.SKIPPED:
                     ti_status.skipped.add(key)
                     self.log.debug("Task instance %s skipped. Don't rerun.", ti)
                     ti_status.to_run.pop(key)
@@ -464,24 +442,17 @@ class BackfillJob(BaseJob):
                         ti_status.running.pop(key)
                     return
 
-                # guard against externally modified tasks instances or
-                # in case max concurrency has been reached at task runtime
-                elif ti.state == State.NONE:
-                    self.log.warning(
-                        "FIXME: Task instance %s state was set to None externally. This should not happen", ti
-                    )
-                    ti.set_state(State.SCHEDULED, session=session)
                 if self.rerun_failed_tasks:
                     # Rerun failed tasks or upstreamed failed tasks
-                    if ti.state in (State.FAILED, State.UPSTREAM_FAILED):
+                    if ti.state in (TaskInstanceState.FAILED, TaskInstanceState.UPSTREAM_FAILED):
                         self.log.error("Task instance %s with state %s", ti, ti.state)
                         if key in ti_status.running:
                             ti_status.running.pop(key)
                         # Reset the failed task in backfill to scheduled state
-                        ti.set_state(State.SCHEDULED, session=session)
+                        ti.set_state(TaskInstanceState.SCHEDULED, session=session)
                 else:
                     # Default behaviour which works for subdag.
-                    if ti.state in (State.FAILED, State.UPSTREAM_FAILED):
+                    if ti.state in (TaskInstanceState.FAILED, TaskInstanceState.UPSTREAM_FAILED):
                         self.log.error("Task instance %s with state %s", ti, ti.state)
                         ti_status.failed.add(key)
                         ti_status.to_run.pop(key)
@@ -512,10 +483,17 @@ class BackfillJob(BaseJob):
                     else:
                         self.log.debug('Sending %s to executor', ti)
                         # Skip scheduled state, we are executing immediately
-                        ti.state = State.QUEUED
+                        ti.state = TaskInstanceState.QUEUED
                         ti.queued_by_job_id = self.id
                         ti.queued_dttm = timezone.utcnow()
                         session.merge(ti)
+                        try:
+                            session.commit()
+                        except OperationalError:
+                            self.log.exception("Failed to commit task state change due to operational error")
+                            session.rollback()
+                            # early exit so the outer loop can retry
+                            return
 
                         cfg_path = None
                         if self.executor_class in (
@@ -535,10 +513,9 @@ class BackfillJob(BaseJob):
                         )
                         ti_status.running[key] = ti
                         ti_status.to_run.pop(key)
-                    session.commit()
                     return
 
-                if ti.state == State.UPSTREAM_FAILED:
+                if ti.state == TaskInstanceState.UPSTREAM_FAILED:
                     self.log.error("Task instance %s upstream failed", ti)
                     ti_status.failed.add(key)
                     ti_status.to_run.pop(key)
@@ -547,7 +524,7 @@ class BackfillJob(BaseJob):
                     return
 
                 # special case
-                if ti.state == State.UP_FOR_RETRY:
+                if ti.state == TaskInstanceState.UP_FOR_RETRY:
                     self.log.debug("Task instance %s retry period not expired yet", ti)
                     if key in ti_status.running:
                         ti_status.running.pop(key)
@@ -555,7 +532,7 @@ class BackfillJob(BaseJob):
                     return
 
                 # special case
-                if ti.state == State.UP_FOR_RESCHEDULE:
+                if ti.state == TaskInstanceState.UP_FOR_RESCHEDULE:
                     self.log.debug("Task instance %s reschedule period not expired yet", ti)
                     if key in ti_status.running:
                         ti_status.running.pop(key)
@@ -579,8 +556,7 @@ class BackfillJob(BaseJob):
                         open_slots = pool.open_slots(session=session)
                         if open_slots <= 0:
                             raise NoAvailablePoolSlot(
-                                "Not scheduling since there are "
-                                "{} open slots in pool {}".format(open_slots, task.pool)
+                                f"Not scheduling since there are {open_slots} open slots in pool {task.pool}"
                             )
 
                         num_running_task_instances_in_dag = DAG.get_num_task_instances(
@@ -607,7 +583,8 @@ class BackfillJob(BaseJob):
                                     "Not scheduling since Task concurrency limit is reached."
                                 )
 
-                        _per_task_process(key, ti)
+                        _per_task_process(key, ti, session)
+                        session.commit()
             except (NoAvailablePoolSlot, DagConcurrencyLimitReached, TaskConcurrencyLimitReached) as e:
                 self.log.debug(e)
 
@@ -627,11 +604,28 @@ class BackfillJob(BaseJob):
                 ti_status.deadlocked.update(ti_status.to_run.values())
                 ti_status.to_run.clear()
 
-            # check executor state
-            self._manage_executor_state(ti_status.running)
+            # check executor state -- and expand any mapped TIs
+            for node, run_id, new_mapped_tis, max_map_index in self._manage_executor_state(
+                ti_status.running, session
+            ):
+
+                def to_keep(key: TaskInstanceKey) -> bool:
+                    if key.dag_id != node.dag_id or key.task_id != node.task_id or key.run_id != run_id:
+                        # For another Dag/Task/Run -- don't remove
+                        return True
+                    return 0 <= key.map_index <= max_map_index
+
+                # remove the old unmapped TIs for node -- they have been replaced with the mapped TIs
+                ti_status.to_run = {key: ti for (key, ti) in ti_status.to_run.items() if to_keep(key)}
+
+                ti_status.to_run.update({ti.key: ti for ti in new_mapped_tis})
+
+                for new_ti in new_mapped_tis:
+                    new_ti.set_state(TaskInstanceState.SCHEDULED, session=session)
 
             # update the task counters
-            self._update_counters(ti_status=ti_status)
+            self._update_counters(ti_status=ti_status, session=session)
+            session.commit()
 
             # update dag run state
             _dag_runs = ti_status.active_runs[:]
@@ -643,25 +637,33 @@ class BackfillJob(BaseJob):
                     executed_run_dates.append(run.execution_date)
 
             self._log_progress(ti_status)
+            session.commit()
 
         # return updated status
         return executed_run_dates
 
     @provide_session
-    def _collect_errors(self, ti_status, session=None):
-        def tabulate_ti_keys_set(set_ti_keys: Set[TaskInstanceKey]) -> str:
+    def _collect_errors(self, ti_status: _DagRunTaskStatus, session=None):
+        def tabulate_ti_keys_set(ti_keys: Iterable[TaskInstanceKey]) -> str:
             # Sorting by execution date first
-            sorted_ti_keys = sorted(
-                set_ti_keys,
-                key=lambda ti_key: (ti_key.run_id, ti_key.dag_id, ti_key.task_id, ti_key.try_number),
+            sorted_ti_keys: Any = sorted(
+                ti_keys,
+                key=lambda ti_key: (
+                    ti_key.run_id,
+                    ti_key.dag_id,
+                    ti_key.task_id,
+                    ti_key.map_index,
+                    ti_key.try_number,
+                ),
             )
-            return tabulate(sorted_ti_keys, headers=["DAG ID", "Task ID", "Run ID", "Try number"])
 
-        def tabulate_tis_set(set_tis: Set[TaskInstance]) -> str:
-            # Sorting by execution date first
-            sorted_tis = sorted(set_tis, key=lambda ti: (ti.run_id, ti.dag_id, ti.task_id, ti.try_number))
-            tis_values = ((ti.dag_id, ti.task_id, ti.run_id, ti.try_number) for ti in sorted_tis)
-            return tabulate(tis_values, headers=["DAG ID", "Task ID", "Run ID", "Try number"])
+            if all(key.map_index == -1 for key in ti_keys):
+                headers = ["DAG ID", "Task ID", "Run ID", "Try number"]
+                sorted_ti_keys = map(lambda k: k[0:4], sorted_ti_keys)
+            else:
+                headers = ["DAG ID", "Task ID", "Run ID", "Map Index", "Try number"]
+
+            return tabulate(sorted_ti_keys, headers=headers)
 
         err = ''
         if ti_status.failed:
@@ -697,10 +699,13 @@ class BackfillJob(BaseJob):
             err += '\n\nThese tasks are skipped:\n'
             err += tabulate_ti_keys_set(ti_status.skipped)
             err += '\n\nThese tasks are deadlocked:\n'
-            err += tabulate_tis_set(ti_status.deadlocked)
+            err += tabulate_ti_keys_set([ti.key for ti in ti_status.deadlocked])
 
         return err
 
+    def _get_dag_with_subdags(self) -> list[DAG]:
+        return [self.dag] + self.dag.subdags
+
     @provide_session
     def _execute_dagruns(self, dagrun_infos, ti_status, executor, pickle_id, start_date, session=None):
         """
@@ -709,22 +714,16 @@ class BackfillJob(BaseJob):
         Returns a list of execution dates of the dag runs that were executed.
 
         :param dagrun_infos: Schedule information for dag runs
-        :type dagrun_infos: list[DagRunInfo]
         :param ti_status: internal BackfillJob status structure to tis track progress
-        :type ti_status: BackfillJob._DagRunTaskStatus
         :param executor: the executor to use, it must be previously started
-        :type executor: BaseExecutor
         :param pickle_id: numeric id of the pickled dag, None if not pickled
-        :type pickle_id: int
         :param start_date: backfill start date
-        :type start_date: datetime.datetime
         :param session: the current session object
-        :type session: sqlalchemy.orm.session.Session
         """
         for dagrun_info in dagrun_infos:
-            for dag in [self.dag] + self.dag.subdags:
+            for dag in self._get_dag_with_subdags():
                 dag_run = self._get_dag_run(dagrun_info, dag, session=session)
-                tis_map = self._task_instances_for_dag_run(dag_run, session=session)
+                tis_map = self._task_instances_for_dag_run(dag, dag_run, session=session)
                 if dag_run is None:
                     continue
 
@@ -754,7 +753,7 @@ class BackfillJob(BaseJob):
         for dag_run in dag_runs:
             dag_run.update_state()
             if dag_run.state not in State.finished:
-                dag_run.set_state(State.FAILED)
+                dag_run.set_state(DagRunState.FAILED)
             session.merge(dag_run)
 
     @provide_session
@@ -778,9 +777,8 @@ class BackfillJob(BaseJob):
             tasks_that_depend_on_past = [t.task_id for t in self.dag.task_dict.values() if t.depends_on_past]
             if tasks_that_depend_on_past:
                 raise AirflowException(
-                    'You cannot backfill backwards because one or more tasks depend_on_past: {}'.format(
-                        ",".join(tasks_that_depend_on_past)
-                    )
+                    f'You cannot backfill backwards because one or more '
+                    f'tasks depend_on_past: {",".join(tasks_that_depend_on_past)}'
                 )
             dagrun_infos = dagrun_infos[::-1]
 
@@ -790,6 +788,31 @@ class BackfillJob(BaseJob):
                 return
             dagrun_infos = [DagRunInfo.interval(dagrun_start_date, dagrun_end_date)]
 
+        dag_with_subdags_ids = [d.dag_id for d in self._get_dag_with_subdags()]
+        running_dagruns = DagRun.find(
+            dag_id=dag_with_subdags_ids,
+            execution_start_date=self.bf_start_date,
+            execution_end_date=self.bf_end_date,
+            no_backfills=True,
+            state=DagRunState.RUNNING,
+        )
+
+        if running_dagruns:
+            for run in running_dagruns:
+                self.log.error(
+                    "Backfill cannot be created for DagRun %s in %s, as there's already %s in a RUNNING "
+                    "state.",
+                    run.run_id,
+                    run.execution_date.strftime("%Y-%m-%dT%H:%M:%S"),
+                    run.run_type,
+                )
+            self.log.error(
+                "Changing DagRun into BACKFILL would cause scheduler to lose track of executing "
+                "tasks. Not changing DagRun type into BACKFILL, and trying insert another DagRun into "
+                "database would cause database constraint violation for dag_id + execution_date "
+                "combination. Please adjust backfill dates or wait for this DagRun to finish.",
+            )
+            return
         # picklin'
         pickle_id = None
 
@@ -829,7 +852,8 @@ class BackfillJob(BaseJob):
                 remaining_dates = ti_status.total_runs - len(ti_status.executed_dag_run_dates)
                 err = self._collect_errors(ti_status=ti_status, session=session)
                 if err:
-                    raise BackfillUnfinished(err, ti_status)
+                    if not self.continue_on_failures or ti_status.deadlocked:
+                        raise BackfillUnfinished(err, ti_status)
 
                 if remaining_dates > 0:
                     self.log.info(
@@ -848,7 +872,7 @@ class BackfillJob(BaseJob):
             session.commit()
             executor.end()
 
-        self.log.info("Backfill done. Exiting.")
+        self.log.info("Backfill done for DAG %s. Exiting.", self.dag)
 
     @provide_session
     def reset_state_for_orphaned_tasks(self, filter_by_dag_run=None, session=None):
@@ -860,7 +884,6 @@ class BackfillJob(BaseJob):
         are made in sequence.
 
         :param filter_by_dag_run: the dag_run we want to process, None if all
-        :type filter_by_dag_run: airflow.models.DagRun
         :return: the number of TIs reset
         :rtype: int
         """
@@ -869,13 +892,13 @@ class BackfillJob(BaseJob):
         running_tis = self.executor.running
 
         # Can't use an update here since it doesn't support joins.
-        resettable_states = [State.SCHEDULED, State.QUEUED]
+        resettable_states = [TaskInstanceState.SCHEDULED, TaskInstanceState.QUEUED]
         if filter_by_dag_run is None:
             resettable_tis = (
                 session.query(TaskInstance)
                 .join(TaskInstance.dag_run)
                 .filter(
-                    DagRun.state == State.RUNNING,
+                    DagRun.state == DagRunState.RUNNING,
                     DagRun.run_type != DagRunType.BACKFILL_JOB,
                     TaskInstance.state.in_(resettable_states),
                 )
