@@ -1,4 +1,3 @@
-#
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
 # distributed with this work for additional information
@@ -15,21 +14,21 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+"""Merge migrations Heads
 
-"""Straighten out the migrations
-
-Revision ID: 08364691d074
-Revises: a56c9515abdc, 004c1210f153, 74effc47d867, b3b105409875
-Create Date: 2019-11-19 22:05:11.752222
+Revision ID: 05f30312d566
+Revises: 86770d1215c0, 0e2a74e0fc9f
+Create Date: 2018-06-17 10:47:23.339972
 
 """
+from __future__ import annotations
 
 # revision identifiers, used by Alembic.
-revision = '08364691d074'
-down_revision = ('a56c9515abdc', '004c1210f153', '74effc47d867', 'b3b105409875')
+revision = '05f30312d566'
+down_revision = ('86770d1215c0', '0e2a74e0fc9f')
 branch_labels = None
 depends_on = None
-airflow_version = '1.10.7'
+airflow_version = '1.10.0'
 
 
 def upgrade():
