@@ -16,21 +16,22 @@
 # specific language governing permissions and limitations
 # under the License.
 """Save Rendered Template Fields"""
+from __future__ import annotations
+
 import os
-from typing import Optional
 
 import sqlalchemy_jsonfield
-from sqlalchemy import Column, String, and_, not_, tuple_
-from sqlalchemy.orm import Session
+from sqlalchemy import Column, ForeignKeyConstraint, Integer, PrimaryKeyConstraint, and_, not_, text, tuple_
+from sqlalchemy.ext.associationproxy import association_proxy
+from sqlalchemy.orm import Session, relationship
 
 from airflow.configuration import conf
-from airflow.models.base import ID_LEN, Base
+from airflow.models.base import Base, StringID
 from airflow.models.taskinstance import TaskInstance
 from airflow.serialization.helpers import serialize_template_field
 from airflow.settings import json
 from airflow.utils.retries import retry_db_transaction
-from airflow.utils.session import provide_session
-from airflow.utils.sqlalchemy import UtcDateTime
+from airflow.utils.session import NEW_SESSION, provide_session
 
 
 class RenderedTaskInstanceFields(Base):
@@ -38,20 +39,62 @@ class RenderedTaskInstanceFields(Base):
 
     __tablename__ = "rendered_task_instance_fields"
 
-    dag_id = Column(String(ID_LEN), primary_key=True)
-    task_id = Column(String(ID_LEN), primary_key=True)
-    execution_date = Column(UtcDateTime, primary_key=True)
+    dag_id = Column(StringID(), primary_key=True)
+    task_id = Column(StringID(), primary_key=True)
+    run_id = Column(StringID(), primary_key=True)
+    map_index = Column(Integer, primary_key=True, server_default=text('-1'))
     rendered_fields = Column(sqlalchemy_jsonfield.JSONField(json=json), nullable=False)
     k8s_pod_yaml = Column(sqlalchemy_jsonfield.JSONField(json=json), nullable=True)
 
+    __table_args__ = (
+        PrimaryKeyConstraint(
+            "dag_id",
+            "task_id",
+            "run_id",
+            "map_index",
+            name='rendered_task_instance_fields_pkey',
+            mssql_clustered=True,
+        ),
+        ForeignKeyConstraint(
+            [dag_id, task_id, run_id, map_index],
+            [
+                "task_instance.dag_id",
+                "task_instance.task_id",
+                "task_instance.run_id",
+                "task_instance.map_index",
+            ],
+            name='rtif_ti_fkey',
+            ondelete="CASCADE",
+        ),
+    )
+    task_instance = relationship(
+        "TaskInstance",
+        lazy='joined',
+        back_populates="rendered_task_instance_fields",
+    )
+
+    # We don't need a DB level FK here, as we already have that to TI (which has one to DR) but by defining
+    # the relationship we can more easily find the execution date for these rows
+    dag_run = relationship(
+        "DagRun",
+        primaryjoin="""and_(
+            RenderedTaskInstanceFields.dag_id == foreign(DagRun.dag_id),
+            RenderedTaskInstanceFields.run_id == foreign(DagRun.run_id),
+        )""",
+        viewonly=True,
+    )
+
+    execution_date = association_proxy("dag_run", "execution_date")
+
     def __init__(self, ti: TaskInstance, render_templates=True):
         self.dag_id = ti.dag_id
         self.task_id = ti.task_id
-        self.task = ti.task
-        self.execution_date = ti.execution_date
+        self.run_id = ti.run_id
+        self.map_index = ti.map_index
         self.ti = ti
         if render_templates:
             ti.render_templates()
+        self.task = ti.task
         if os.environ.get("AIRFLOW_IS_K8S_EXECUTOR_POD", None):
             self.k8s_pod_yaml = ti.render_k8s_pod_yaml()
         self.rendered_fields = {
@@ -61,7 +104,10 @@ class RenderedTaskInstanceFields(Base):
         self._redact()
 
     def __repr__(self):
-        return f"<{self.__class__.__name__}: {self.dag_id}.{self.task_id} {self.execution_date}"
+        prefix = f"<{self.__class__.__name__}: {self.dag_id}.{self.task_id} {self.run_id}"
+        if self.map_index != -1:
+            prefix += f" map_index={self.map_index}"
+        return prefix + '>'
 
     def _redact(self):
         from airflow.utils.log.secrets_masker import redact
@@ -74,7 +120,7 @@ class RenderedTaskInstanceFields(Base):
 
     @classmethod
     @provide_session
-    def get_templated_fields(cls, ti: TaskInstance, session: Session = None) -> Optional[dict]:
+    def get_templated_fields(cls, ti: TaskInstance, session: Session = NEW_SESSION) -> dict | None:
         """
         Get templated field for a TaskInstance from the RenderedTaskInstanceFields
         table.
@@ -86,7 +132,10 @@ class RenderedTaskInstanceFields(Base):
         result = (
             session.query(cls.rendered_fields)
             .filter(
-                cls.dag_id == ti.dag_id, cls.task_id == ti.task_id, cls.execution_date == ti.execution_date
+                cls.dag_id == ti.dag_id,
+                cls.task_id == ti.task_id,
+                cls.run_id == ti.run_id,
+                cls.map_index == ti.map_index,
             )
             .one_or_none()
         )
@@ -99,7 +148,7 @@ class RenderedTaskInstanceFields(Base):
 
     @classmethod
     @provide_session
-    def get_k8s_pod_yaml(cls, ti: TaskInstance, session: Session = None) -> Optional[dict]:
+    def get_k8s_pod_yaml(cls, ti: TaskInstance, session: Session = NEW_SESSION) -> dict | None:
         """
         Get rendered Kubernetes Pod Yaml for a TaskInstance from the RenderedTaskInstanceFields
         table.
@@ -111,7 +160,10 @@ class RenderedTaskInstanceFields(Base):
         result = (
             session.query(cls.k8s_pod_yaml)
             .filter(
-                cls.dag_id == ti.dag_id, cls.task_id == ti.task_id, cls.execution_date == ti.execution_date
+                cls.dag_id == ti.dag_id,
+                cls.task_id == ti.task_id,
+                cls.run_id == ti.run_id,
+                cls.map_index == ti.map_index,
             )
             .one_or_none()
         )
@@ -135,31 +187,38 @@ class RenderedTaskInstanceFields(Base):
         session: Session = None,
     ):
         """
-        Keep only Last X (num_to_keep) number of records for a task by deleting others
+        Keep only Last X (num_to_keep) number of records for a task by deleting others.
+
+        In the case of data for a mapped task either all of the rows or none of the rows will be deleted, so
+        we don't end up with partial data for a set of mapped Task Instances left in the database.
 
         :param task_id: Task ID
         :param dag_id: Dag ID
         :param num_to_keep: Number of Records to keep
         :param session: SqlAlchemy Session
         """
+        from airflow.models.dagrun import DagRun
+
         if num_to_keep <= 0:
             return
 
         tis_to_keep_query = (
-            session.query(cls.dag_id, cls.task_id, cls.execution_date)
+            session.query(cls.dag_id, cls.task_id, cls.run_id)
             .filter(cls.dag_id == dag_id, cls.task_id == task_id)
-            .order_by(cls.execution_date.desc())
+            .join(cls.dag_run)
+            .distinct()
+            .order_by(DagRun.execution_date.desc())
             .limit(num_to_keep)
         )
 
         if session.bind.dialect.name in ["postgresql", "sqlite"]:
             # Fetch Top X records given dag_id & task_id ordered by Execution Date
-            subq1 = tis_to_keep_query.subquery('subq1')
-
+            subq1 = tis_to_keep_query.subquery()
+            excluded = session.query(subq1.c.dag_id, subq1.c.task_id, subq1.c.run_id)
             session.query(cls).filter(
                 cls.dag_id == dag_id,
                 cls.task_id == task_id,
-                tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq1),
+                tuple_(cls.dag_id, cls.task_id, cls.run_id).notin_(excluded),
             ).delete(synchronize_session=False)
         elif session.bind.dialect.name in ["mysql"]:
             cls._remove_old_rendered_ti_fields_mysql(dag_id, session, task_id, tis_to_keep_query)
@@ -172,7 +231,7 @@ class RenderedTaskInstanceFields(Base):
                     and_(
                         cls.dag_id == ti.dag_id,
                         cls.task_id == ti.task_id,
-                        cls.execution_date == ti.execution_date,
+                        cls.run_id == ti.run_id,
                     )
                 )
                 for ti in tis_to_keep
@@ -180,6 +239,8 @@ class RenderedTaskInstanceFields(Base):
 
             session.query(cls).filter(and_(*filter_tis)).delete(synchronize_session=False)
 
+        session.flush()
+
     @classmethod
     @retry_db_transaction
     def _remove_old_rendered_ti_fields_mysql(cls, dag_id, session, task_id, tis_to_keep_query):
@@ -189,11 +250,10 @@ class RenderedTaskInstanceFields(Base):
         # Workaround for MySQL Limitation (https://stackoverflow.com/a/19344141/5691525)
         # Limitation: This version of MySQL does not yet support
         # LIMIT & IN/ALL/ANY/SOME subquery
-        subq2 = session.query(subq1.c.dag_id, subq1.c.task_id, subq1.c.execution_date).subquery('subq2')
+        subq2 = session.query(subq1.c.dag_id, subq1.c.task_id, subq1.c.run_id).subquery('subq2')
         # This query might deadlock occasionally and it should be retried if fails (see decorator)
         session.query(cls).filter(
             cls.dag_id == dag_id,
             cls.task_id == task_id,
-            tuple_(cls.dag_id, cls.task_id, cls.execution_date).notin_(subq2),
+            tuple_(cls.dag_id, cls.task_id, cls.run_id).notin_(subq2),
         ).delete(synchronize_session=False)
-        session.flush()
