@@ -16,6 +16,8 @@
 # specific language governing permissions and limitations
 # under the License.
 """Objects relating to retrieving connections and variables from local file"""
+from __future__ import annotations
+
 import json
 import logging
 import os
@@ -23,13 +25,14 @@ import warnings
 from collections import defaultdict
 from inspect import signature
 from json import JSONDecodeError
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Set, Tuple
+from typing import TYPE_CHECKING, Any
 
 from airflow.exceptions import (
     AirflowException,
     AirflowFileParseException,
     ConnectionNotUnique,
     FileSyntaxError,
+    RemovedInAirflow3Warning,
 )
 from airflow.secrets.base_secrets import BaseSecretsBackend
 from airflow.utils import yaml
@@ -42,14 +45,14 @@ if TYPE_CHECKING:
     from airflow.models.connection import Connection
 
 
-def get_connection_parameter_names() -> Set[str]:
+def get_connection_parameter_names() -> set[str]:
     """Returns :class:`airflow.models.connection.Connection` constructor parameters."""
     from airflow.models.connection import Connection
 
     return {k for k in signature(Connection.__init__).parameters.keys() if k != "self"}
 
 
-def _parse_env_file(file_path: str) -> Tuple[Dict[str, List[str]], List[FileSyntaxError]]:
+def _parse_env_file(file_path: str) -> tuple[dict[str, list[str]], list[FileSyntaxError]]:
     """
     Parse a file in the ``.env`` format.
 
@@ -58,14 +61,13 @@ def _parse_env_file(file_path: str) -> Tuple[Dict[str, List[str]], List[FileSynt
         MY_CONN_ID=my-conn-type://my-login:my-pa%2Fssword@my-host:5432/my-schema?param1=val1&param2=val2
 
     :param file_path: The location of the file that will be processed.
-    :type file_path: str
     :return: Tuple with mapping of key and list of values and list of syntax errors
     """
     with open(file_path) as f:
         content = f.read()
 
-    secrets: Dict[str, List[str]] = defaultdict(list)
-    errors: List[FileSyntaxError] = []
+    secrets: dict[str, list[str]] = defaultdict(list)
+    errors: list[FileSyntaxError] = []
     for line_no, line in enumerate(content.splitlines(), 1):
         if not line:
             # Ignore empty line
@@ -96,12 +98,11 @@ def _parse_env_file(file_path: str) -> Tuple[Dict[str, List[str]], List[FileSynt
     return secrets, errors
 
 
-def _parse_yaml_file(file_path: str) -> Tuple[Dict[str, List[str]], List[FileSyntaxError]]:
+def _parse_yaml_file(file_path: str) -> tuple[dict[str, list[str]], list[FileSyntaxError]]:
     """
     Parse a file in the YAML format.
 
     :param file_path: The location of the file that will be processed.
-    :type file_path: str
     :return: Tuple with mapping of key and list of values and list of syntax errors
     """
     with open(file_path) as f:
@@ -111,21 +112,20 @@ def _parse_yaml_file(file_path: str) -> Tuple[Dict[str, List[str]], List[FileSyn
         return {}, [FileSyntaxError(line_no=1, message="The file is empty.")]
     try:
         secrets = yaml.safe_load(content)
-
     except yaml.MarkedYAMLError as e:
-        return {}, [FileSyntaxError(line_no=e.problem_mark.line, message=str(e))]
+        err_line_no = e.problem_mark.line if e.problem_mark else -1
+        return {}, [FileSyntaxError(line_no=err_line_no, message=str(e))]
     if not isinstance(secrets, dict):
         return {}, [FileSyntaxError(line_no=1, message="The file should contain the object.")]
 
     return secrets, []
 
 
-def _parse_json_file(file_path: str) -> Tuple[Dict[str, Any], List[FileSyntaxError]]:
+def _parse_json_file(file_path: str) -> tuple[dict[str, Any], list[FileSyntaxError]]:
     """
     Parse a file in the JSON format.
 
     :param file_path: The location of the file that will be processed.
-    :type file_path: str
     :return: Tuple with mapping of key and list of values and list of syntax errors
     """
     with open(file_path) as f:
@@ -147,15 +147,15 @@ FILE_PARSERS = {
     "env": _parse_env_file,
     "json": _parse_json_file,
     "yaml": _parse_yaml_file,
+    "yml": _parse_yaml_file,
 }
 
 
-def _parse_secret_file(file_path: str) -> Dict[str, Any]:
+def _parse_secret_file(file_path: str) -> dict[str, Any]:
     """
     Based on the file extension format, selects a parser, and parses the file.
 
     :param file_path: The location of the file that will be processed.
-    :type file_path: str
     :return: Map of secret key (e.g. connection ID) and value.
     """
     if not os.path.exists(file_path):
@@ -169,7 +169,8 @@ def _parse_secret_file(file_path: str) -> Dict[str, Any]:
 
     if ext not in FILE_PARSERS:
         raise AirflowException(
-            "Unsupported file format. The file must have the extension .env or .json or .yaml"
+            "Unsupported file format. The file must have one of the following extensions: "
+            ".env .json .yaml .yml"
         )
 
     secrets, parse_errors = FILE_PARSERS[ext](file_path)
@@ -222,14 +223,13 @@ def _create_connection(conn_id: str, value: Any):
     )
 
 
-def load_variables(file_path: str) -> Dict[str, str]:
+def load_variables(file_path: str) -> dict[str, str]:
     """
     Load variables from a text file.
 
     ``JSON``, `YAML` and ``.env`` files are supported.
 
     :param file_path: The location of the file that will be processed.
-    :type file_path: str
     :rtype: Dict[str, List[str]]
     """
     log.debug("Loading variables from a text file")
@@ -243,17 +243,17 @@ def load_variables(file_path: str) -> Dict[str, str]:
     return variables
 
 
-def load_connections(file_path) -> Dict[str, List[Any]]:
+def load_connections(file_path) -> dict[str, list[Any]]:
     """This function is deprecated. Please use `airflow.secrets.local_filesystem.load_connections_dict`.","""
     warnings.warn(
         "This function is deprecated. Please use `airflow.secrets.local_filesystem.load_connections_dict`.",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
         stacklevel=2,
     )
     return {k: [v] for k, v in load_connections_dict(file_path).values()}
 
 
-def load_connections_dict(file_path: str) -> Dict[str, Any]:
+def load_connections_dict(file_path: str) -> dict[str, Any]:
     """
     Load connection from text file.
 
@@ -264,7 +264,7 @@ def load_connections_dict(file_path: str) -> Dict[str, Any]:
     """
     log.debug("Loading connection")
 
-    secrets: Dict[str, Any] = _parse_secret_file(file_path)
+    secrets: dict[str, Any] = _parse_secret_file(file_path)
     connection_by_conn_id = {}
     for key, secret_values in list(secrets.items()):
         if isinstance(secret_values, list):
@@ -289,20 +289,16 @@ class LocalFilesystemBackend(BaseSecretsBackend, LoggingMixin):
     ``JSON``, `YAML` and ``.env`` files are supported.
 
     :param variables_file_path: File location with variables data.
-    :type variables_file_path: str
     :param connections_file_path: File location with connection data.
-    :type connections_file_path: str
     """
 
-    def __init__(
-        self, variables_file_path: Optional[str] = None, connections_file_path: Optional[str] = None
-    ):
+    def __init__(self, variables_file_path: str | None = None, connections_file_path: str | None = None):
         super().__init__()
         self.variables_file = variables_file_path
         self.connections_file = connections_file_path
 
     @property
-    def _local_variables(self) -> Dict[str, str]:
+    def _local_variables(self) -> dict[str, str]:
         if not self.variables_file:
             self.log.debug("The file for variables is not specified. Skipping")
             # The user may not specify any file.
@@ -311,23 +307,23 @@ class LocalFilesystemBackend(BaseSecretsBackend, LoggingMixin):
         return secrets
 
     @property
-    def _local_connections(self) -> Dict[str, 'Connection']:
+    def _local_connections(self) -> dict[str, Connection]:
         if not self.connections_file:
             self.log.debug("The file for connection is not specified. Skipping")
             # The user may not specify any file.
             return {}
         return load_connections_dict(self.connections_file)
 
-    def get_connection(self, conn_id: str) -> Optional['Connection']:
+    def get_connection(self, conn_id: str) -> Connection | None:
         if conn_id in self._local_connections:
             return self._local_connections[conn_id]
         return None
 
-    def get_connections(self, conn_id: str) -> List[Any]:
+    def get_connections(self, conn_id: str) -> list[Any]:
         warnings.warn(
             "This method is deprecated. Please use "
             "`airflow.secrets.local_filesystem.LocalFilesystemBackend.get_connection`.",
-            PendingDeprecationWarning,
+            RemovedInAirflow3Warning,
             stacklevel=2,
         )
         conn = self.get_connection(conn_id=conn_id)
@@ -335,5 +331,5 @@ class LocalFilesystemBackend(BaseSecretsBackend, LoggingMixin):
             return [conn]
         return []
 
-    def get_variable(self, key: str) -> Optional[str]:
+    def get_variable(self, key: str) -> str | None:
         return self._local_variables.get(key)
