@@ -15,6 +15,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 from airflow.exceptions import AirflowException
 from airflow.ti_deps.deps.base_ti_dep import BaseTIDep
@@ -27,7 +28,6 @@ class ValidStateDep(BaseTIDep):
 
     :param valid_states: A list of valid states that a task instance can have to meet
         this dependency.
-    :type valid_states: set(str)
     :return: whether or not the task instance's state is valid
     """
 
