@@ -15,6 +15,8 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import contextlib
 import enum
 import logging
@@ -24,7 +26,7 @@ import time
 import warnings
 from dataclasses import dataclass
 from tempfile import gettempdir
-from typing import TYPE_CHECKING, Callable, Generator, Iterable, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Callable, Generator, Iterable
 
 from sqlalchemy import Table, and_, column, exc, func, inspect, or_, select, table, text, tuple_
 from sqlalchemy.orm.session import Session
@@ -34,44 +36,19 @@ from airflow import settings
 from airflow.compat.sqlalchemy import has_table
 from airflow.configuration import conf
 from airflow.exceptions import AirflowException
-from airflow.jobs.base_job import BaseJob  # noqa: F401
-from airflow.models import (  # noqa: F401
-    DAG,
-    XCOM_RETURN_KEY,
-    Base,
-    BaseOperator,
-    BaseOperatorLink,
-    Connection,
-    DagBag,
-    DagModel,
-    DagPickle,
-    DagRun,
-    DagTag,
-    Log,
-    Pool,
-    SkipMixin,
-    SlaMiss,
-    TaskFail,
-    TaskInstance,
-    TaskReschedule,
-    Variable,
-    XCom,
-)
-
-# We need to add this model manually to get reset working well
-from airflow.models.serialized_dag import SerializedDagModel  # noqa: F401
-from airflow.models.tasklog import LogTemplate
+from airflow.models import import_all_models
 from airflow.utils import helpers
 
 # TODO: remove create_session once we decide to break backward compatibility
 from airflow.utils.session import NEW_SESSION, create_session, provide_session  # noqa: F401
-from airflow.version import version
 
 if TYPE_CHECKING:
     from alembic.runtime.environment import EnvironmentContext
     from alembic.script import ScriptDirectory
     from sqlalchemy.orm import Query
 
+    from airflow.models.base import Base
+
 
 log = logging.getLogger(__name__)
 
@@ -94,6 +71,8 @@ REVISION_HEADS_MAP = {
     "2.3.1": "1de7bc13c950",
     "2.3.2": "3c94c427fdf6",
     "2.3.3": "f5fcbda3e651",
+    "2.3.4": "f5fcbda3e651",
+    "2.4.0": "ecb43d2a1842",
 }
 
 
@@ -104,7 +83,7 @@ def _format_airflow_moved_table_name(source_table, version, category):
 @provide_session
 def merge_conn(conn, session: Session = NEW_SESSION):
     """Add new Connection."""
-    if not session.query(Connection).filter(Connection.conn_id == conn.conn_id).first():
+    if not session.query(conn.__class__).filter_by(conn_id=conn.conn_id).first():
         session.add(conn)
         session.commit()
 
@@ -112,6 +91,8 @@ def merge_conn(conn, session: Session = NEW_SESSION):
 @provide_session
 def add_default_pool_if_not_exists(session: Session = NEW_SESSION):
     """Add default pool if it does not exist."""
+    from airflow.models.pool import Pool
+
     if not Pool.get_pool(Pool.DEFAULT_POOL_NAME, session=session):
         default_pool = Pool(
             pool=Pool.DEFAULT_POOL_NAME,
@@ -125,6 +106,8 @@ def add_default_pool_if_not_exists(session: Session = NEW_SESSION):
 @provide_session
 def create_default_connections(session: Session = NEW_SESSION):
     """Create default Airflow connections."""
+    from airflow.models.connection import Connection
+
     merge_conn(
         Connection(
             conn_id="airflow_db",
@@ -534,6 +517,16 @@ def create_default_connections(session: Session = NEW_SESSION):
         ),
         session,
     )
+    merge_conn(
+        Connection(
+            conn_id="salesforce_default",
+            conn_type="salesforce",
+            login="username",
+            password="password",
+            extra='{"security_token": "security_token"}',
+        ),
+        session,
+    )
     merge_conn(
         Connection(
             conn_id="segment_default",
@@ -597,6 +590,14 @@ def create_default_connections(session: Session = NEW_SESSION):
         ),
         session,
     )
+    merge_conn(
+        Connection(
+            conn_id="tabular_default",
+            conn_type="tabular",
+            host="https://api.tabulardata.io/ws/v1",
+        ),
+        session,
+    )
     merge_conn(
         Connection(
             conn_id="trino_default",
@@ -643,19 +644,47 @@ def create_default_connections(session: Session = NEW_SESSION):
     )
 
 
-@provide_session
-def initdb(session: Session = NEW_SESSION):
-    """Initialize Airflow database."""
-    upgradedb(session=session)
+def _create_db_from_orm(session):
+    from alembic import command
+    from flask import Flask
+    from flask_sqlalchemy import SQLAlchemy
 
-    if conf.getboolean('database', 'LOAD_DEFAULT_CONNECTIONS'):
-        create_default_connections(session=session)
+    from airflow.models.base import Base
+    from airflow.www.fab_security.sqla.models import Model
+    from airflow.www.session import AirflowDatabaseSessionInterface
+
+    def _create_flask_session_tbl():
+        flask_app = Flask(__name__)
+        flask_app.config['SQLALCHEMY_DATABASE_URI'] = conf.get('database', 'SQL_ALCHEMY_CONN')
+        db = SQLAlchemy(flask_app)
+        AirflowDatabaseSessionInterface(app=flask_app, db=db, table='session', key_prefix='')
+        db.create_all()
 
     with create_global_lock(session=session, lock=DBLocks.MIGRATIONS):
-
-        from flask_appbuilder.models.sqla import Base
-
         Base.metadata.create_all(settings.engine)
+        Model.metadata.create_all(settings.engine)
+        _create_flask_session_tbl()
+        # stamp the migration head
+        config = _get_alembic_config()
+        command.stamp(config, "head")
+
+
+@provide_session
+def initdb(session: Session = NEW_SESSION, load_connections: bool = True):
+    """Initialize Airflow database."""
+    import_all_models()
+
+    db_exists = _get_current_revision(session)
+    if db_exists:
+        upgradedb(session=session)
+    else:
+        _create_db_from_orm(session=session)
+    # Load default connections
+    if conf.getboolean('database', 'LOAD_DEFAULT_CONNECTIONS') and load_connections:
+        create_default_connections(session=session)
+    # Add default pool & sync log_template
+    add_default_pool_if_not_exists()
+    synchronize_log_template()
 
 
 def _get_alembic_config():
@@ -669,7 +698,7 @@ def _get_alembic_config():
     return config
 
 
-def _get_script_object(config=None) -> "ScriptDirectory":
+def _get_script_object(config=None) -> ScriptDirectory:
     from alembic.script import ScriptDirectory
 
     if not config:
@@ -712,7 +741,7 @@ def check_migrations(timeout):
 
 
 @contextlib.contextmanager
-def _configured_alembic_environment() -> Generator["EnvironmentContext", None, None]:
+def _configured_alembic_environment() -> Generator[EnvironmentContext, None, None]:
     from alembic.runtime.environment import EnvironmentContext
 
     config = _get_alembic_config()
@@ -760,6 +789,8 @@ def check_and_run_migrations():
                     db_command()
                     print(f"DB {verb} done")
                 except Exception as error:
+                    from airflow.version import version
+
                     print(error)
                     print(
                         "You still have unapplied migrations. "
@@ -771,6 +802,8 @@ def check_and_run_migrations():
         except AirflowException:
             pass
     elif source_heads != db_heads:
+        from airflow.version import version
+
         print(
             f"ERROR: You need to {verb} the database. Please run `airflow db {command_name}`. "
             f"Make sure the command is run using Airflow version {version}.",
@@ -779,6 +812,17 @@ def check_and_run_migrations():
         sys.exit(1)
 
 
+@provide_session
+def reserialize_dags(*, session: Session = NEW_SESSION) -> None:
+    from airflow.models.dagbag import DagBag
+    from airflow.models.serialized_dag import SerializedDagModel
+
+    session.query(SerializedDagModel).delete(synchronize_session=False)
+    dagbag = DagBag()
+    dagbag.collect_dags(only_if_updated=False, safe_mode=False)
+    dagbag.sync_to_db(session=session)
+
+
 @provide_session
 def synchronize_log_template(*, session: Session = NEW_SESSION) -> None:
     """Synchronize log template configs with table.
@@ -786,6 +830,7 @@ def synchronize_log_template(*, session: Session = NEW_SESSION) -> None:
     This checks if the last row fully matches the current config values, and
     insert a new row if not.
     """
+    from airflow.models.tasklog import LogTemplate
 
     def log_template_exists():
         metadata = reflect_tables([LogTemplate], session)
@@ -855,6 +900,8 @@ def check_conn_id_duplicates(session: Session) -> Iterable[str]:
     :param session:  session of the sqlalchemy
     :rtype: str
     """
+    from airflow.models.connection import Connection
+
     dups = []
     try:
         dups = session.query(Connection.conn_id).group_by(Connection.conn_id).having(func.count() > 1).all()
@@ -870,7 +917,7 @@ def check_conn_id_duplicates(session: Session) -> Iterable[str]:
         )
 
 
-def reflect_tables(tables: List[Union[Base, str]], session):
+def reflect_tables(tables: list[Base | str] | None, session):
     """
     When running checks prior to upgrades, we use reflection to determine current state of the
     database.
@@ -881,17 +928,22 @@ def reflect_tables(tables: List[Union[Base, str]], session):
 
     metadata = sqlalchemy.schema.MetaData(session.bind)
 
-    for tbl in tables:
-        try:
-            table_name = tbl if isinstance(tbl, str) else tbl.__tablename__
-            metadata.reflect(only=[table_name], extend_existing=True, resolve_fks=False)
-        except exc.InvalidRequestError:
-            continue
+    if tables is None:
+        metadata.reflect(resolve_fks=False)
+    else:
+        for tbl in tables:
+            try:
+                table_name = tbl if isinstance(tbl, str) else tbl.__tablename__
+                metadata.reflect(only=[table_name], extend_existing=True, resolve_fks=False)
+            except exc.InvalidRequestError:
+                continue
     return metadata
 
 
 def check_task_fail_for_duplicates(session):
     """Check that there are no duplicates in the task_fail table before creating FK"""
+    from airflow.models.taskfail import TaskFail
+
     metadata = reflect_tables([TaskFail], session)
     task_fail = metadata.tables.get(TaskFail.__tablename__)  # type: ignore
     if task_fail is None:  # table not there
@@ -907,7 +959,7 @@ def check_task_fail_for_duplicates(session):
 
 
 def check_table_for_duplicates(
-    *, session: Session, table_name: str, uniqueness: List[str], version: str
+    *, session: Session, table_name: str, uniqueness: list[str], version: str
 ) -> Iterable[str]:
     """
     Check table for duplicates, given a list of columns which define the uniqueness of the table.
@@ -961,6 +1013,8 @@ def check_conn_type_null(session: Session) -> Iterable[str]:
     :param session:  session of the sqlalchemy
     :rtype: str
     """
+    from airflow.models.connection import Connection
+
     n_nulls = []
     try:
         n_nulls = session.query(Connection.conn_id).filter(Connection.conn_type.is_(None)).all()
@@ -990,6 +1044,8 @@ def _format_dangling_error(source_table, target_table, invalid_count, reason):
 
 
 def check_run_id_null(session: Session) -> Iterable[str]:
+    from airflow.models.dagrun import DagRun
+
     metadata = reflect_tables([DagRun], session)
 
     # We can't use the model here since it may differ from the db state due to
@@ -1003,7 +1059,7 @@ def check_run_id_null(session: Session) -> Iterable[str]:
         dagrun_table.c.run_id.is_(None),
         dagrun_table.c.execution_date.is_(None),
     )
-    invalid_dagrun_count = session.query(dagrun_table.c.id).filter(invalid_dagrun_filter).count()
+    invalid_dagrun_count = session.query(func.count(dagrun_table.c.id)).filter(invalid_dagrun_filter).scalar()
     if invalid_dagrun_count > 0:
         dagrun_dangling_table_name = _format_airflow_moved_table_name(dagrun_table.name, '2.2', 'dangling')
         if dagrun_dangling_table_name in inspect(session.get_bind()).get_table_names():
@@ -1032,7 +1088,7 @@ def _create_table_as(
     *,
     session,
     dialect_name: str,
-    source_query: "Query",
+    source_query: Query,
     target_table_name: str,
     source_table_name: str,
 ):
@@ -1066,7 +1122,7 @@ def _create_table_as(
 
 
 def _move_dangling_data_to_new_table(
-    session, source_table: "Table", source_query: "Query", target_table_name: str
+    session, source_table: Table, source_query: Query, target_table_name: str
 ):
 
     bind = session.get_bind()
@@ -1171,7 +1227,7 @@ def _dangling_against_task_instance(session, source_table, dag_run, task_instanc
 
 
 def _move_duplicate_data_to_new_table(
-    session, source_table: "Table", subquery: "Query", uniqueness: List[str], target_table_name: str
+    session, source_table: Table, subquery: Query, uniqueness: list[str], target_table_name: str
 ):
     """
     When adding a uniqueness constraint we first should ensure that there are no duplicate rows.
@@ -1229,7 +1285,12 @@ def check_bad_references(session: Session) -> Iterable[str]:
     When we find such "dangling" rows we back them up in a special table and delete them
     from the main table.
     """
+    from airflow.models.dagrun import DagRun
     from airflow.models.renderedtifields import RenderedTaskInstanceFields
+    from airflow.models.taskfail import TaskFail
+    from airflow.models.taskinstance import TaskInstance
+    from airflow.models.taskreschedule import TaskReschedule
+    from airflow.models.xcom import XCom
 
     @dataclass
     class BadReferenceConfig:
@@ -1240,7 +1301,7 @@ def check_bad_references(session: Session) -> Iterable[str]:
         """
 
         bad_rows_func: Callable
-        join_tables: List[str]
+        join_tables: list[str]
         ref_table: str
 
     missing_dag_run_config = BadReferenceConfig(
@@ -1255,7 +1316,7 @@ def check_bad_references(session: Session) -> Iterable[str]:
         ref_table='task_instance',
     )
 
-    models_list: List[Tuple[Base, str, BadReferenceConfig]] = [
+    models_list: list[tuple[Base, str, BadReferenceConfig]] = [
         (TaskInstance, '2.2', missing_dag_run_config),
         (TaskReschedule, '2.2', missing_ti_config),
         (RenderedTaskInstanceFields, '2.3', missing_ti_config),
@@ -1325,7 +1386,7 @@ def _check_migration_errors(session: Session = NEW_SESSION) -> Iterable[str]:
     :session: session of the sqlalchemy
     :rtype: list[str]
     """
-    check_functions: Tuple[Callable[..., Iterable[str]], ...] = (
+    check_functions: tuple[Callable[..., Iterable[str]], ...] = (
         check_task_fail_for_duplicates,
         check_conn_id_duplicates,
         check_conn_type_null,
@@ -1400,8 +1461,8 @@ def _revisions_above_min_for_offline(config, revisions):
 @provide_session
 def upgradedb(
     *,
-    to_revision: Optional[str] = None,
-    from_revision: Optional[str] = None,
+    to_revision: str | None = None,
+    from_revision: str | None = None,
     show_sql_only: bool = False,
     session: Session = NEW_SESSION,
 ):
@@ -1424,6 +1485,8 @@ def upgradedb(
         raise RuntimeError("The settings.SQL_ALCHEMY_CONN not set. This is a critical assertion.")
     from alembic import command
 
+    import_all_models()
+
     config = _get_alembic_config()
 
     if show_sql_only:
@@ -1458,11 +1521,17 @@ def upgradedb(
     if errors_seen:
         exit(1)
 
+    if not to_revision and not _get_current_revision(session=session):
+        # Don't load default connections
+        # New DB; initialize and exit
+        initdb(session=session, load_connections=False)
+        return
     with create_global_lock(session=session, lock=DBLocks.MIGRATIONS):
         log.info("Creating tables")
         command.upgrade(config, revision=to_revision or 'heads')
-    add_default_pool_if_not_exists()
-    synchronize_log_template()
+    reserialize_dags(session=session)
+    add_default_pool_if_not_exists(session=session)
+    synchronize_log_template(session=session)
 
 
 @provide_session
@@ -1472,6 +1541,8 @@ def resetdb(session: Session = NEW_SESSION, skip_init: bool = False):
         raise RuntimeError("The settings.engine must be set. This is a critical assertion")
     log.info("Dropping tables that exist")
 
+    import_all_models()
+
     connection = settings.engine.connect()
 
     with create_global_lock(session=session, lock=DBLocks.MIGRATIONS):
@@ -1485,6 +1556,8 @@ def resetdb(session: Session = NEW_SESSION, skip_init: bool = False):
 
 @provide_session
 def bootstrap_dagbag(session: Session = NEW_SESSION):
+    from airflow.models.dag import DAG
+    from airflow.models.dagbag import DagBag
 
     dagbag = DagBag()
     # Save DAGs in the ORM
@@ -1579,7 +1652,8 @@ def drop_airflow_moved_tables(session):
     tables = set(inspect(session.get_bind()).get_table_names())
     to_delete = [Table(x, Base.metadata) for x in tables if x.startswith(AIRFLOW_MOVED_TABLE_PREFIX)]
     for tbl in to_delete:
-        tbl.drop(settings.engine, checkfirst=True)
+        tbl.drop(settings.engine, checkfirst=False)
+        Base.metadata.remove(tbl)
 
 
 def drop_flask_models(connection):
@@ -1654,6 +1728,64 @@ def create_global_lock(
             pass
 
 
+def compare_type(context, inspected_column, metadata_column, inspected_type, metadata_type):
+    """
+    Compare types between ORM and DB .
+
+    return False if the metadata_type is the same as the inspected_type
+    or None to allow the default implementation to compare these
+    types. a return value of True means the two types do not
+    match and should result in a type change operation.
+    """
+    if context.dialect.name == 'mysql':
+        from sqlalchemy import String
+        from sqlalchemy.dialects import mysql
+
+        if isinstance(inspected_type, mysql.VARCHAR) and isinstance(metadata_type, String):
+            # This is a hack to get around MySQL VARCHAR collation
+            # not being possible to change from utf8_bin to utf8mb3_bin.
+            # We only make sure lengths are the same
+            if inspected_type.length != metadata_type.length:
+                return True
+            return False
+    return None
+
+
+def compare_server_default(
+    context, inspected_column, metadata_column, inspected_default, metadata_default, rendered_metadata_default
+):
+    """
+    Compare server defaults between ORM and DB .
+
+    return True if the defaults are different, False if not, or None to allow the default implementation
+    to compare these defaults
+
+    Comparing server_default is not accurate in MSSQL because the
+    inspected_default above != metadata_default, while in Postgres/MySQL they are equal.
+    This is an issue with alembic
+    In SQLite: task_instance.map_index & task_reschedule.map_index
+    are not comparing accurately. Sometimes they are equal, sometimes they are not.
+    Alembic warned that this feature has varied accuracy depending on backends.
+    See: (https://alembic.sqlalchemy.org/en/latest/api/runtime.html#alembic.runtime.
+        environment.EnvironmentContext.configure.params.compare_server_default)
+    """
+    dialect_name = context.connection.dialect.name
+    if dialect_name in ['mssql', 'sqlite']:
+        return False
+    if (
+        dialect_name == 'mysql'
+        and metadata_column.name == 'pool_slots'
+        and metadata_column.table.name == 'task_instance'
+    ):
+        # We removed server_default value in ORM to avoid expensive migration
+        # (it was removed in postgres DB in migration head 7b2661a43ba3 ).
+        # As a side note, server default value here was only actually needed for the migration
+        # where we added the column in the first place -- now that it exists and all
+        # existing rows are populated with a value this server default is never used.
+        return False
+    return None
+
+
 def get_sqla_model_classes():
     """
     Get all SQLAlchemy class mappers.
