@@ -14,10 +14,13 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Serve logs process"""
+from __future__ import annotations
+
+import collections
 import logging
 import os
+import socket
 
 import gunicorn.app.base
 from flask import Flask, abort, request, send_from_directory
@@ -108,6 +111,9 @@ def create_app():
     return flask_app
 
 
+GunicornOption = collections.namedtuple("GunicornOption", ["key", "value"])
+
+
 class StandaloneGunicornApplication(gunicorn.app.base.BaseApplication):
     """
     Standalone Gunicorn application/serve for usage with any WSGI-application.
@@ -120,13 +126,13 @@ class StandaloneGunicornApplication(gunicorn.app.base.BaseApplication):
     """
 
     def __init__(self, app, options=None):
-        self.options = options or {}
+        self.options = options or []
         self.application = app
         super().__init__()
 
     def load_config(self):
-        for key, value in self.options.items():
-            self.cfg.set(key.lower(), value)
+        for option in self.options:
+            self.cfg.set(option.key.lower(), option.value)
 
     def load(self):
         return self.application
@@ -138,10 +144,15 @@ def serve_logs():
     wsgi_app = create_app()
 
     worker_log_server_port = conf.getint('logging', 'WORKER_LOG_SERVER_PORT')
-    options = {
-        'bind': f"0.0.0.0:{worker_log_server_port}",
-        'workers': 2,
-    }
+
+    # If dual stack is available and IPV6_V6ONLY is not enabled on the socket
+    # then when IPV6 is bound to it will also bind to IPV4 automatically
+    if getattr(socket, "has_dualstack_ipv6", lambda: False)():
+        bind_option = GunicornOption("bind", f"[::]:{worker_log_server_port}")
+    else:
+        bind_option = GunicornOption("bind", f"0.0.0.0:{worker_log_server_port}")
+
+    options = [bind_option, GunicornOption("workers", 2)]
     StandaloneGunicornApplication(wsgi_app, options).run()
 
 
