@@ -15,19 +15,23 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
+from __future__ import annotations
+
 import warnings
 from datetime import timedelta
-from typing import Optional
+from tempfile import gettempdir
 
 from flask import Flask
 from flask_appbuilder import SQLA
 from flask_caching import Cache
 from flask_wtf.csrf import CSRFProtect
+from sqlalchemy.engine.url import make_url
 
 from airflow import settings
 from airflow.configuration import conf
+from airflow.exceptions import AirflowConfigException, RemovedInAirflow3Warning
 from airflow.logging_config import configure_logging
+from airflow.models import import_all_models
 from airflow.utils.json import AirflowJsonEncoder
 from airflow.www.extensions.init_appbuilder import init_appbuilder
 from airflow.www.extensions.init_appbuilder_links import init_appbuilder_links
@@ -36,7 +40,7 @@ from airflow.www.extensions.init_jinja_globals import init_jinja_globals
 from airflow.www.extensions.init_manifest_files import configure_manifest_files
 from airflow.www.extensions.init_robots import init_robots
 from airflow.www.extensions.init_security import init_api_experimental_auth, init_xframe_protection
-from airflow.www.extensions.init_session import init_airflow_session_interface, init_permanent_session
+from airflow.www.extensions.init_session import init_airflow_session_interface
 from airflow.www.extensions.init_views import (
     init_api_connexion,
     init_api_experimental,
@@ -48,7 +52,7 @@ from airflow.www.extensions.init_views import (
 )
 from airflow.www.extensions.init_wsgi_middlewares import init_wsgi_middleware
 
-app: Optional[Flask] = None
+app: Flask | None = None
 
 # Initializes at the module level, so plugins can access it.
 # See: /docs/plugins.rst
@@ -74,7 +78,15 @@ def create_app(config=None, testing=False):
     flask_app.config.from_pyfile(settings.WEBSERVER_CONFIG, silent=True)
     flask_app.config['APP_NAME'] = conf.get(section="webserver", key="instance_name", fallback="Airflow")
     flask_app.config['TESTING'] = testing
-    flask_app.config['SQLALCHEMY_DATABASE_URI'] = conf.get('core', 'SQL_ALCHEMY_CONN')
+    flask_app.config['SQLALCHEMY_DATABASE_URI'] = conf.get('database', 'SQL_ALCHEMY_CONN')
+
+    url = make_url(flask_app.config['SQLALCHEMY_DATABASE_URI'])
+    if url.drivername == 'sqlite' and url.database and not url.database.startswith('/'):
+        raise AirflowConfigException(
+            f'Cannot use relative path: `{conf.get("database", "SQL_ALCHEMY_CONN")}` to connect to sqlite. '
+            'Please use absolute path such as `sqlite:////tmp/airflow.db`.'
+        )
+
     flask_app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
 
     flask_app.config['SESSION_COOKIE_HTTPONLY'] = True
@@ -85,7 +97,7 @@ def create_app(config=None, testing=False):
         warnings.warn(
             "Old deprecated value found for `cookie_samesite` option in `[webserver]` section. "
             "Using `Lax` instead. Change the value to `Lax` in airflow.cfg to remove this warning.",
-            DeprecationWarning,
+            RemovedInAirflow3Warning,
         )
         cookie_samesite_config = "Lax"
     flask_app.config['SESSION_COOKIE_SAMESITE'] = cookie_samesite_config
@@ -97,7 +109,7 @@ def create_app(config=None, testing=False):
         flask_app.config['SQLALCHEMY_ENGINE_OPTIONS'] = settings.prepare_engine_args()
 
     # Configure the JSON encoder used by `|tojson` filter from Flask
-    flask_app.json_encoder = AirflowJsonEncoder
+    flask_app.json_provider_class = AirflowJsonEncoder
 
     csrf.init_app(flask_app)
 
@@ -113,13 +125,16 @@ def create_app(config=None, testing=False):
 
     init_robots(flask_app)
 
-    Cache(app=flask_app, config={'CACHE_TYPE': 'filesystem', 'CACHE_DIR': '/tmp'})
+    cache_config = {'CACHE_TYPE': 'flask_caching.backends.filesystem', 'CACHE_DIR': gettempdir()}
+    Cache(app=flask_app, config=cache_config)
 
     init_flash_views(flask_app)
 
     configure_logging()
     configure_manifest_files(flask_app)
 
+    import_all_models()
+
     with flask_app.app_context():
         init_appbuilder(flask_app)
 
@@ -135,7 +150,6 @@ def create_app(config=None, testing=False):
 
         init_jinja_globals(flask_app)
         init_xframe_protection(flask_app)
-        init_permanent_session(flask_app)
         init_airflow_session_interface(flask_app)
     return flask_app
 
